/*
   ipc.c -- PIKT ipc (interprocess communication) routines

   $Header: /var/cvs/PIKT/pikt/src/lib/ipc.c,v 1.13 2004/01/29 15:06:08 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001,2002,2003,2004 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org

   ----------------------------------------------------------------------------

   We might redo the striplex()-to-*parse() coordination using signals, but
   IPC and pipes are more portable.

 */

#include "common.h"

int p2cfd[MAXIFS+1][2], c2pfd[MAXIFS+1][2];

/* open the parent-to-child and child-to-parent channels
 */
void
ipcopen (int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "ipcopen";

	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "pid is %d, ppid is %d, incnum is %d, called from %s, line %d", getpid(), getppid(), incnum, srcfile, lno);

	if (pipe(p2cfd[incnum]) < 0 || pipe(c2pfd[incnum]) < 0)
		logexit(PIKTLOG, LOG_ERR, ERRID_PIPE_ERROR, __FILE__, __LINE__, fnnam, TRUE, "pipe(p2cfd[%d]) or pipe(c2pfd[%d]) error", incnum, incnum);
	return;
}

/* close the parent-to-child and child-to-parent channels
 */
void
ipcclose(int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "ipcclose";

	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "pid is %d, ppid is %d, incnum is %d, called from %s, line %d", getpid(), getppid(), incnum, srcfile, lno);

	close(p2cfd[incnum][0]);
	close(p2cfd[incnum][1]);
	close(c2pfd[incnum][0]);
	close(c2pfd[incnum][1]);
	return;
}

/* the child process signals the parent
 */
void
tell_parent (int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "tell_parent";

	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "pid is %d, ppid is %d, incnum is %d, called from %s, line %d", getpid(), getppid(), incnum, srcfile, lno);

	if (write(c2pfd[incnum][1], "c", 1) != 1)
		logreturn(PIKTLOG, LOG_ERR, ERRID_PIPE_WRITE_ERROR, __FILE__, __LINE__, fnnam, FALSE, "(called from %s, line %d), write(c2pfd[%d][1]) error", srcfile, lno, incnum);
	return;
}

/* the child process waits for the parent's signal
 */
void
wait_parent (int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "wait_parent";
	static char c;
	int n;

	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "pid is %d, ppid is %d, incnum is %d, called from %s, line %d", getpid(), getppid(), incnum, srcfile, lno);

	while (((n = read(p2cfd[incnum][0], &c, 1)) != 1) && (n != 0))
		sleep(1);
	if ((n == 1) && (c != 'p'))
		logreturn(PIKTLOG, LOG_ERR, ERRID_IPC_SIGNAL_INVALID, __FILE__, __LINE__, fnnam, FALSE, "(called from %s, line %d), invalid signal: %c", srcfile, lno, c);
	return;
}

/* the parent process signals the child
 */
void
tell_child (int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "tell_child";

	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "pid is %d, ppid is %d, incnum is %d, called from %s, line %d", getpid(), getppid(), incnum, srcfile, lno);

	if (write(p2cfd[incnum][1], "p", 1) != 1)
		logreturn(PIKTLOG, LOG_ERR, ERRID_PIPE_WRITE_ERROR, __FILE__, __LINE__, fnnam, FALSE, "(called from %s, line %d), write(p2cfd[%d][1]) error", srcfile, lno, incnum);
	return;
}

/* the parent process waits for the child's signal; we allow for the
   possibility that the child has already died, perhaps closing down the
   channel
 */
void
wait_child (int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "wait_child";
	static char c;
	int n;

	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, ERRID_DEBUG, __FILE__, __LINE__, fnnam, FALSE, "pid is %d, ppid is %d, incnum is %d, called from %s, line %d", getpid(), getppid(), incnum, srcfile, lno);

	while (((n = read(c2pfd[incnum][0], &c, 1)) != 1) && (n != 0))
		sleep(1);
	if ((n == 1) && (c != 'c'))
		logreturn(PIKTLOG, LOG_ERR, ERRID_IPC_SIGNAL_INVALID, __FILE__, __LINE__, fnnam, FALSE, "(called from %s, line %d), invalid signal: %c", srcfile, lno, c);
	return;
}
